@@ -1,4 +1,4 @@
-/* Copyright (C) 2016 Chris Vine
+/* Copyright (C) 2016 and 2020 Chris Vine
 
 The library comprised in this file or of which this file is part is
 distributed by Chris Vine under the GNU Lesser General Public
@@ -67,14 +67,14 @@ of the source code in this file).
  * fixed buffer size (the AsyncChannel implementation uses a circular
  * buffer with in-buffer construction of data items).
  *
- * This class is available from version 2.0.31.
+ * This class is available from version 2.2.14.
  */
 
 #ifndef CGU_ASYNC_CHANNEL_H
 #define CGU_ASYNC_CHANNEL_H
 
 #include <utility>    // for std::move and std::forward
-#include <new>        // for std::bad_alloc
+#include <new>        // for std::bad_alloc and __cpp_lib_launder
 #include <cstddef>    // for std::size_t
 #include <cstdlib>    // for std::malloc
 
@@ -137,7 +137,7 @@ inline void cgu_async_channel_waiters_dec(void* arg) {
  * the data items to be placed on the queue.  'n' is the size of the
  * channel, which as mentioned may be any size greater than 0.
  *
- * This class is available from version 2.0.31.
+ * This class is available from version 2.2.14.
  */
 
 /*
@@ -242,9 +242,9 @@ public:
  * the consumer thread can abandon and destroy the AsyncChannel
  * object.
  *
- * Since 2.0.31
+ * Since 2.0.31 and 2.2.14
  */
-  void close() {
+  void close() noexcept {
     Thread::Mutex::Lock lock{mutex};
     if (status == normal) {
       status = closed;
@@ -273,7 +273,7 @@ public:
  * this method are made after the channel has been closed, this method
  * returns false.
  *
- * Since 2.0.31
+ * Since 2.0.31 and 2.2.14
  */
   bool push(const value_type& obj) {
     bool waiting = false;
@@ -334,7 +334,7 @@ public:
  * this method are made after the channel has been closed, this method
  * returns false.
  *
- * Since 2.0.31
+ * Since 2.0.31 and 2.2.14
  */
   bool push(value_type&& obj) {
     bool waiting = false;
@@ -396,7 +396,7 @@ public:
  * this method are made after the channel has been closed, this method
  * returns false.
  *
- * Since 2.0.31
+ * Since 2.0.31 and 2.2.14
  */
   template<class... Args>
   bool emplace(Args&&... args) {
@@ -459,7 +459,7 @@ public:
  * method are made, and there are no remaining items in the channel,
  * this method returns false.
  *
- * Since 2.0.31
+ * Since 2.0.31 and 2.2.14
  */
   bool pop(value_type& obj) {
     bool waiting = false;
@@ -491,13 +491,20 @@ public:
       return false;
     }
     else if (size) { // status == normal, or status == closed && size > 0
-      size_type old_idx = idx;
-      obj = buf[old_idx];
+      // if this library is employed in a program compiled in c++17 or
+      // greater, permit objects with references or const members to
+      // be stored in AsyncChannel objects via std::launder
+#if defined(__cpp_lib_launder) && __cpp_lib_launder >= 201606
+      T* ptr = std::launder(buf + idx);
+#else
+      T* ptr = buf + idx;
+#endif
+      obj = *ptr;
       ++idx;
       if (idx == n) idx = 0;
       --size;
       try {
-	buf[old_idx].~T();
+	ptr->~T();
       }
       catch (...) {
 	// we might as well keep the AsyncChannel object's waiters in
@@ -549,7 +556,7 @@ public:
  * method are made, and there are no remaining items in the channel,
  * this method returns false.
  *
- * Since 2.0.31
+ * Since 2.0.31 and 2.2.14
  */
   bool move_pop(value_type& obj) {
     bool waiting = false;
@@ -581,13 +588,20 @@ public:
       return false;
     }
     else if (size) { // status == normal, or status == closed && size > 0
-      size_type old_idx = idx;
-      obj = std::move(buf[old_idx]);
+      // if this library is employed in a program compiled in c++17 or
+      // greater, permit objects with references or const members to
+      // be stored in AsyncChannel objects via std::launder
+#if defined(__cpp_lib_launder) && __cpp_lib_launder >= 201606
+      T* ptr = std::launder(buf + idx);
+#else
+      T* ptr = buf + idx;
+#endif
+      obj = std::move(*ptr);
       ++idx;
       if (idx == n) idx = 0;
       --size;
       try {
-	buf[old_idx].~T();
+	ptr->~T();
       }
       catch (...) {
 	// we might as well keep the AsyncChannel object's waiters in
@@ -631,7 +645,7 @@ public:
  * means either memory is exhausted or pthread has run out of other
  * resources to create new condition variables.)
  *
- * Since 2.0.31
+ * Since 2.0.31 and 2.2.14
  */
   AsyncChannel(): size(0), idx(0), waiters(0), status(normal),
                   // locality would be better if we could use an
@@ -659,7 +673,7 @@ public:
  * destructor will not return until all threads (if any) blocking on
  * the AsyncChannel object have been released.
  *
- * Since 2.0.31
+ * Since 2.0.31 and 2.2.14
  */
   ~AsyncChannel() {
     mutex.lock();
@@ -687,7 +701,12 @@ public:
       }
     }
     while (size) {
-      buf[idx].~T();
+#if defined(__cpp_lib_launder) && __cpp_lib_launder >= 201606
+      T* ptr = std::launder(buf + idx);
+#else
+      T* ptr = buf + idx;
+#endif
+      ptr->~T();
       ++idx;
       if (idx == n) idx = 0;
       --size;
@@ -727,7 +746,7 @@ public:
 
   AsyncChannel& operator=(const AsyncChannel&) = delete;
 
-  void close() {
+  void close() noexcept {
     Thread::Mutex::Lock lock{mutex};
     if (status == normal) {
       status = closed;
@@ -871,10 +890,18 @@ public:
       return false;
     }
     else if (full) { // status == normal, or status == closed && full
-      obj = *datum;
+      // if this library is employed in a program compiled in c++17 or
+      // greater, permit objects with references or const members to
+      // be stored in AsyncChannel objects via std::launder
+#if defined(__cpp_lib_launder) && __cpp_lib_launder >= 201606
+      T* ptr = std::launder(datum);
+#else
+      T* ptr = datum;
+#endif
+      obj = *ptr;
       full = false;
       try {
-	datum->~T();
+	ptr->~T();
       }
       catch (...) {
 	// we might as well keep the AsyncChannel object's waiters in
@@ -923,10 +950,15 @@ public:
       return false;
     }
     else if (full) { // status == normal, or status == closed && full
-      obj = std::move(*datum);
+#if defined(__cpp_lib_launder) && __cpp_lib_launder >= 201606
+      T* ptr = std::launder(datum);
+#else
+      T* ptr = datum;
+#endif
+      obj = std::move(*ptr);
       full = false;
       try {
-	datum->~T();
+	ptr->~T();
       }
       catch (...) {
 	// we might as well keep the AsyncChannel object's waiters in
@@ -984,7 +1016,11 @@ public:
 	break;
       }
     }
+#if defined(__cpp_lib_launder) && __cpp_lib_launder >= 201606
+    if (full) std::launder(datum)->~T();
+#else
     if (full) datum->~T();
+#endif
     std::free(datum);
   }
 
