@@ -100,12 +100,6 @@ namespace Cgu {}
 */
 #define CGU_HAVE_RECURSIVE_MUTEX 1
 
-/* whether to use std::auto_ptr instead of std::unique_ptr in relevant
-   public interfaces to maintain source compatibility with earlier
-   versions
-*/
-#undef CGU_USE_AUTO_PTR
-
 /* whether ==, != and < operators to be available for smart pointers */
 #define CGU_USE_SMART_PTR_COMPARISON 1
 
@@ -114,7 +108,7 @@ namespace Cgu {}
 #define CGU_USE_INHERITABLE_QUEUE 1
 
 /* whether the library is to be compiled with GTK+ support */
-#define CGU_USE_GTK 1
+#define CGU_USE_GTK 3
 
 /* whether the library is to be compiled with support for guile
    extensions */
@@ -123,12 +117,15 @@ namespace Cgu {}
 /* whether guile correctly links scm_dynwind_block_asyncs() */
 #undef CGU_GUILE_HAS_BROKEN_LINKING
 
+/* whether to use std::tuple implementation of callback classes */
+#define CGU_USE_TUPLE 1
+
 /* whether gcc diagnostic pragmas are available */
 #define CGU_USE_DIAGNOSTIC_PRAGMAS 1
 
 /* indicates that the series of the library being compiled against
-   is series 2.0
+   is series 2.2
 */
-#define CGU_API_VERSION 20
+#define CGU_API_VERSION 22
 
 #endif // CGU_CONFIG_H
