@@ -1,4 +1,4 @@
-/* Copyright (C) 2006, 2007, 2011 and 2013 to 2019 Chris Vine
+/* Copyright (C) 2006, 2007, 2011 and 2013 to 2020 Chris Vine
 
 The library comprised in this file or of which this file is part is
 distributed by Chris Vine under the GNU Lesser General Public
@@ -86,18 +86,14 @@ FilePrintDialog::FilePrintDialog(GtkWindow* parent_p, GtkPrintSettings* print_se
 					    true, parent_p,
 					    (GtkWindow*)gtk_print_unix_dialog_new(0, 0)) {
 
-#if GTK_CHECK_VERSION(3,94,0)
-  gtk_window_set_type_hint(get_win(), GDK_SURFACE_TYPE_HINT_DIALOG);
-#else
+#if !(GTK_CHECK_VERSION(3,94,0))
   gtk_window_set_type_hint(get_win(), GDK_WINDOW_TYPE_HINT_DIALOG);
+  gtk_window_set_position(get_win(), GTK_WIN_POS_CENTER_ON_PARENT);
 #endif
 
   g_signal_connect((GObject*)get_win(), "response",
 		   G_CALLBACK(cgu_fpd_selected), this);
 
-  gtk_window_set_position(get_win(), GTK_WIN_POS_CENTER_ON_PARENT);
-  gtk_window_set_resizable(get_win(), false);
-
   if (print_settings_p) {
     gtk_print_unix_dialog_set_settings((GtkPrintUnixDialog*)get_win(), print_settings_p);
   }
