@@ -64,9 +64,9 @@ struct ParallelError: public std::exception {
 
 #ifndef DOXYGEN_PARSING
 
-// in version 2.0.19, this was the ParallelHelper namespace.  Because
+// in version 2.2.2, this was the ParallelHelper namespace.  Because
 // the meaning of the DiffType* argument of these functions has
-// changed in version 2.0.20 (it is incremented rather than
+// changed in version 2.2.3 (it is incremented rather than
 // decremented), this is now the ParallelHelper2 namespace so that no
 // ODR issues arise on library linking with old binaries
 namespace ParallelHelper2 {
@@ -138,17 +138,13 @@ void transform2_cb_func(const Cgu::Callback::SafeFunctorArg<Arg1RefType, Arg2Ref
 
 template <class DiffType>
 void fail_func(Mutex* m, Cond* cond,
-	       bool* error, DiffType* done_count) {
+	       bool* error, DiffType* done_count) noexcept {
   Mutex::Lock l{*m};
   ++*done_count;
   *error = true;
   cond->signal();
 }
 
-inline void fail_cb_func(const Cgu::Callback::SafeFunctor& s_fail) {
-  s_fail(); 
-}
-
 } // namespace ParallelHelper2
 
 #endif // DOXYGEN_PARSING
@@ -246,10 +242,11 @@ inline void fail_cb_func(const Cgu::Callback::SafeFunctor& s_fail) {
  * @note 1. An exception might also be thrown if the copy or move
  * constructor of the 'func' callable objects throws.  If such an
  * exception is thrown, no tasks will start.
- * @note 2. Prior to version 2.0.27, this function could not take a
- * source iterator to const.  This was fixed in version 2.0.27.
+ * @note 2. Prior to version 2.0.27 and 2.2.10, this function could
+ * not take a source iterator to const.  This was fixed in versions
+ * 2.0.27 and 2.2.10.
  * 
- * Since 2.0.19
+ * Since 2.0.19/2.2.2
  */
 template <class Iterator, class Func>
 void parallel_for_each(TaskManager& tm,
@@ -280,13 +277,7 @@ void parallel_for_each(TaskManager& tm,
   };
 
   for (; first != last; ++first, ++start_count) {
-#ifdef CGU_USE_AUTO_PTR
-    typedef std::auto_ptr<const Callback::Callback> CbPtr;
-#else
-    typedef std::unique_ptr<const Callback::Callback> CbPtr;
-#endif
-
-    CbPtr task_cb(
+    std::unique_ptr<const Cgu::Callback::Callback> task_cb(
       Cgu::Callback::make_ref(&ParallelHelper2::for_each_cb_func<ArgRefType, DiffType, Iterator>,
 			      s_task,
 			      first,
@@ -294,8 +285,8 @@ void parallel_for_each(TaskManager& tm,
 			      &cond,
 			      &done_count)
     );
-    CbPtr fail_cb(
-      Cgu::Callback::make_ref(&ParallelHelper2::fail_cb_func, s_fail)
+    std::unique_ptr<const Cgu::Callback::Callback> fail_cb(
+      Cgu::Callback::lambda<>([s_fail] () {s_fail();})
     );
 
     tm.add_task(std::move(task_cb), std::move(fail_cb));
@@ -330,6 +321,11 @@ void parallel_for_each(TaskManager& tm,
  * range).  It will not return until the callable object has been
  * applied to all of the elements in the range ['first', 'last').
  *
+ * Where the objects to be stored in the destination container are
+ * heavyweight non-trivial objects of class type, for best performance
+ * they should have an efficient move constructor and move assignment
+ * operator.
+ *
  * This function can be called by a task running on the same
  * TaskManager object, perhaps with a view to delivering a result
  * asynchronously to a glib main loop.  However if that is done, as
@@ -436,10 +432,11 @@ void parallel_for_each(TaskManager& tm,
  * @note 1. An exception might also be thrown if the copy or move
  * constructor of the 'func' callable objects throws.  If such an
  * exception is thrown, no tasks will start.
- * @note 2. Prior to version 2.0.27, this function could not take a
- * source iterator to const.  This was fixed in version 2.0.27.
+ * @note 2. Prior to version 2.0.27 and 2.2.10, this function could
+ * not take a source iterator to const.  This was fixed in versions
+ * 2.0.27 and 2.2.10.
  * 
- * Since 2.0.19
+ * Since 2.0.19/2.2.2
  */
 template <class SourceIterator, class DestIterator, class Func>
 void parallel_transform(TaskManager& tm,
@@ -485,12 +482,7 @@ void parallel_transform(TaskManager& tm,
   };
 
   for (; first != last; ++first, ++start_count) {
-#ifdef CGU_USE_AUTO_PTR
-    typedef std::auto_ptr<const Callback::Callback> CbPtr;
-#else
-    typedef std::unique_ptr<const Callback::Callback> CbPtr;
-#endif
-    CbPtr task_cb(
+    std::unique_ptr<const Cgu::Callback::Callback> task_cb(
       Cgu::Callback::lambda<>(std::bind(&ParallelHelper2::transform1_cb_func<ArgRefType, DestType, DiffType, SourceIterator>,
 					s_task,
 					first,
@@ -499,8 +491,8 @@ void parallel_transform(TaskManager& tm,
 					&done_count,
 					results.get() + start_count))
     );
-    CbPtr fail_cb(
-      Cgu::Callback::make_ref(&ParallelHelper2::fail_cb_func, s_fail)
+    std::unique_ptr<const Cgu::Callback::Callback> fail_cb(
+      Cgu::Callback::lambda<>([s_fail] () {s_fail();})
     );
 
     tm.add_task(std::move(task_cb), std::move(fail_cb));
@@ -539,6 +531,11 @@ void parallel_transform(TaskManager& tm,
  * It will not return until the callable object has been applied to
  * all of the elements in the source ranges.
  *
+ * Where the objects to be stored in the destination container are
+ * heavyweight non-trivial objects of class type, for best performance
+ * they should have an efficient move constructor and move assignment
+ * operator.
+ *
  * This function can be called by a task running on the same
  * TaskManager object, perhaps with a view to delivering a result
  * asynchronously to a glib main loop.  However if that is done, as
@@ -649,10 +646,11 @@ void parallel_transform(TaskManager& tm,
  * @note 1. An exception might also be thrown if the copy or move
  * constructor of the 'func' callable objects throws.  If such an
  * exception is thrown, no tasks will start.
- * @note 2. Prior to version 2.0.27, this function could not take a
- * source iterator to const.  This was fixed in version 2.0.27.
+ * @note 2. Prior to version 2.0.27 and 2.2.10, this function could
+ * not take a source iterator to const.  This was fixed in versions
+ * 2.0.27 and 2.2.10.
  * 
- * Since 2.0.19
+ * Since 2.0.19/2.2.2
  */
 template <class SourceIterator1, class SourceIterator2, class DestIterator, class Func>
 void parallel_transform(TaskManager& tm,
@@ -700,12 +698,7 @@ void parallel_transform(TaskManager& tm,
   };
 
   for (; first1 != last1; ++first1, ++first2, ++start_count) {
-#ifdef CGU_USE_AUTO_PTR
-    typedef std::auto_ptr<const Callback::Callback> CbPtr;
-#else
-    typedef std::unique_ptr<const Callback::Callback> CbPtr;
-#endif
-    CbPtr task_cb(
+    std::unique_ptr<const Cgu::Callback::Callback> task_cb(
       Cgu::Callback::lambda<>(std::bind(&ParallelHelper2::transform2_cb_func<Arg1RefType, Arg2RefType, DestType, DiffType, SourceIterator1, SourceIterator2>,
 					s_task,
 					first1,
@@ -715,8 +708,8 @@ void parallel_transform(TaskManager& tm,
 					&done_count,
 					results.get() + start_count))
     );
-    CbPtr fail_cb(
-      Cgu::Callback::make_ref(&ParallelHelper2::fail_cb_func, s_fail)
+    std::unique_ptr<const Cgu::Callback::Callback> fail_cb(
+      Cgu::Callback::lambda<>([s_fail] () {s_fail();})
     );
 
     tm.add_task(std::move(task_cb), std::move(fail_cb));
@@ -843,10 +836,11 @@ void parallel_transform(TaskManager& tm,
  * @note 1. An exception might also be thrown if the copy or move
  * constructor of the 'func' callable objects throws.  If such an
  * exception is thrown, no tasks will start.
- * @note 2. Prior to version 2.0.27, this function could not take a
- * source iterator to const.  This was fixed in version 2.0.27.
+ * @note 2. Prior to version 2.0.27 and 2.2.10, this function could
+ * not take a source iterator to const.  This was fixed in versions
+ * 2.0.27 and 2.2.10.
  * 
- * Since 2.0.20
+ * Since 2.0.20/2.2.3
  */
 template <class Iterator, class Func>
 Iterator parallel_for_each_partial(TaskManager& tm,
@@ -888,12 +882,7 @@ Iterator parallel_for_each_partial(TaskManager& tm,
   };
 
   for (; first != last && start_count < local_max; ++first, ++start_count) {
-#ifdef CGU_USE_AUTO_PTR
-    typedef std::auto_ptr<const Callback::Callback> CbPtr;
-#else
-    typedef std::unique_ptr<const Callback::Callback> CbPtr;
-#endif
-    CbPtr task_cb(
+    std::unique_ptr<const Cgu::Callback::Callback> task_cb(
       Cgu::Callback::make_ref(&ParallelHelper2::for_each_cb_func<ArgRefType, DiffType, Iterator>,
 			      s_task,
 			      first,
@@ -901,8 +890,8 @@ Iterator parallel_for_each_partial(TaskManager& tm,
 			      &cond,
 			      &done_count)
     );
-    CbPtr fail_cb(
-      Cgu::Callback::make_ref(&ParallelHelper2::fail_cb_func, s_fail)
+    std::unique_ptr<const Cgu::Callback::Callback> fail_cb(
+      Cgu::Callback::lambda<>([s_fail] () {s_fail();})
     );
 
     tm.add_task(std::move(task_cb), std::move(fail_cb));
@@ -1046,10 +1035,11 @@ Iterator parallel_for_each_partial(TaskManager& tm,
  * @note 1. An exception might also be thrown if the copy or move
  * constructor of the 'func' callable objects throws.  If such an
  * exception is thrown, no tasks will start.
- * @note 2. Prior to version 2.0.27, this function could not take a
- * source iterator to const.  This was fixed in version 2.0.27.
+ * @note 2. Prior to version 2.0.27 and 2.2.10, this function could
+ * not take a source iterator to const.  This was fixed in versions
+ * 2.0.27 and 2.2.10.
  * 
- * Since 2.0.20
+ * Since 2.0.20/2.2.3
  */
 template <class SourceIterator, class DestIterator, class Func>
 std::pair<SourceIterator, DestIterator>
@@ -1106,12 +1096,7 @@ std::pair<SourceIterator, DestIterator>
   };
 
   for (; first != last && start_count < local_max; ++first, ++start_count) {
-#ifdef CGU_USE_AUTO_PTR
-    typedef std::auto_ptr<const Callback::Callback> CbPtr;
-#else
-    typedef std::unique_ptr<const Callback::Callback> CbPtr;
-#endif
-    CbPtr task_cb(
+    std::unique_ptr<const Cgu::Callback::Callback> task_cb(
       Cgu::Callback::lambda<>(std::bind(&ParallelHelper2::transform1_cb_func<ArgRefType, DestType, DiffType, SourceIterator>,
 					s_task,
 					first,
@@ -1120,8 +1105,8 @@ std::pair<SourceIterator, DestIterator>
 					&done_count,
 					results.get() + start_count))
     );
-    CbPtr fail_cb(
-      Cgu::Callback::make_ref(&ParallelHelper2::fail_cb_func, s_fail)
+    std::unique_ptr<const Cgu::Callback::Callback> fail_cb(
+      Cgu::Callback::lambda<>([s_fail] () {s_fail();})
     );
 
     tm.add_task(std::move(task_cb), std::move(fail_cb));
@@ -1276,10 +1261,11 @@ std::pair<SourceIterator, DestIterator>
  * @note 1. An exception might also be thrown if the copy or move
  * constructor of the 'func' callable objects throws.  If such an
  * exception is thrown, no tasks will start.
- * @note 2. Prior to version 2.0.27, this function could not take a
- * source iterator to const.  This was fixed in version 2.0.27.
+ * @note 2. Prior to version 2.0.27 and 2.2.10, this function could
+ * not take a source iterator to const.  This was fixed in versions
+ * 2.0.27 and 2.2.10.
  * 
- * Since 2.0.20
+ * Since 2.0.20/2.2.3
  */
 template <class SourceIterator1, class SourceIterator2, class DestIterator, class Func>
 std::tuple<SourceIterator1, SourceIterator2, DestIterator>
@@ -1338,12 +1324,7 @@ std::tuple<SourceIterator1, SourceIterator2, DestIterator>
   };
 
   for (; first1 != last1 && start_count < local_max; ++first1, ++first2, ++start_count) {
-#ifdef CGU_USE_AUTO_PTR
-    typedef std::auto_ptr<const Callback::Callback> CbPtr;
-#else
-    typedef std::unique_ptr<const Callback::Callback> CbPtr;
-#endif
-    CbPtr task_cb(
+    std::unique_ptr<const Cgu::Callback::Callback> task_cb(
       Cgu::Callback::lambda<>(std::bind(&ParallelHelper2::transform2_cb_func<Arg1RefType, Arg2RefType, DestType, DiffType, SourceIterator1, SourceIterator2>,
 					s_task,
 					first1,
@@ -1353,8 +1334,8 @@ std::tuple<SourceIterator1, SourceIterator2, DestIterator>
 					&done_count,
 					results.get() + start_count))
     );
-    CbPtr fail_cb(
-      Cgu::Callback::make_ref(&ParallelHelper2::fail_cb_func, s_fail)
+    std::unique_ptr<const Cgu::Callback::Callback> fail_cb(
+      Cgu::Callback::lambda<>([s_fail] () {s_fail();})
     );
 
     tm.add_task(std::move(task_cb), std::move(fail_cb));
@@ -1443,7 +1424,7 @@ std::tuple<SourceIterator1, SourceIterator2, DestIterator>
  *
  * @ingroup IntIterHelpers
  *
- * Since 2.0.27.
+ * Since 2.0.27 and 2.2.10.
  */
 class IntIter {
 public:
@@ -1459,15 +1440,16 @@ public:
    * This constructor acts as both a default constructor and an
    * initializing constructor.  It does not throw.
    *
-   * Since 2.0.27.
+   * Since 2.0.27 and 2.2.10.
    */
-  explicit IntIter(value_type val_ = 0): val(val_) {}
+  explicit IntIter(value_type val_ = 0) noexcept : val(val_) {}
 
   /**
    * The copy constructor does not throw.
    *
-   * Since 2.0.27.
+   * Since 2.0.27 and 2.2.10.
    */
+  // gcc-4.6 will error if we make this noexcept
   IntIter(const IntIter&) = default;
   
   /**
@@ -1482,8 +1464,9 @@ public:
    * use of an IntIter object only by one of those functions does not
    * require synchronization).
    *
-   * Since 2.0.27.
+   * Since 2.0.27 and 2.2.10.
    */
+  // gcc-4.6 will error if we make this noexcept
   IntIter& operator=(const IntIter&) = default;
   
   /**
@@ -1499,9 +1482,9 @@ public:
    * require synchronization).
    * @return A reference to the iterator after being incremented.
    *
-   * Since 2.0.27.
+   * Since 2.0.27 and 2.2.10.
    */
-  IntIter& operator++() {++val; return *this;}
+  IntIter& operator++() noexcept {++val; return *this;}
 
   /**
    * The post-increment operator does not throw.  No locking is
@@ -1516,9 +1499,9 @@ public:
    * require synchronization).
    * @return A copy of the iterator prior to being incremented.
    *
-   * Since 2.0.27.
+   * Since 2.0.27 and 2.2.10.
    */
-  IntIter operator++(int) {IntIter tmp = *this; ++val; return tmp;}
+  IntIter operator++(int) noexcept {IntIter tmp = *this; ++val; return tmp;}
 
   /**
    * The pre-decrement operator does not throw.  No locking is carried
@@ -1533,9 +1516,9 @@ public:
    * require synchronization).
    * @return A reference to the iterator after being decremented.
    *
-   * Since 2.0.27.
+   * Since 2.0.27 and 2.2.10.
    */
-  IntIter& operator--() {--val; return *this;}
+  IntIter& operator--() noexcept {--val; return *this;}
 
   /**
    * The post-decrement operator does not throw.  No locking is
@@ -1550,9 +1533,9 @@ public:
    * require synchronization).
    * @return A copy of the iterator prior to being decremented.
    *
-   * Since 2.0.27.
+   * Since 2.0.27 and 2.2.10.
    */
-  IntIter operator--(int) {IntIter tmp = *this; --val; return tmp;}
+  IntIter operator--(int) noexcept {IntIter tmp = *this; --val; return tmp;}
 
   /**
    * This operator adds the value of the argument to the integer value
@@ -1568,9 +1551,9 @@ public:
    * require synchronization).
    * @return A reference to the iterator after addition.
    *
-   * Since 2.0.27.
+   * Since 2.0.27 and 2.2.10.
    */
-  IntIter& operator+=(difference_type n) {val += n; return *this;}
+  IntIter& operator+=(difference_type n) noexcept {val += n; return *this;}
 
   /**
    * This operator subtracts the value of the argument from the
@@ -1586,9 +1569,9 @@ public:
    * require synchronization).
    * @return A reference to the iterator after subtraction.
    *
-   * Since 2.0.27.
+   * Since 2.0.27 and 2.2.10.
    */
-  IntIter& operator-=(difference_type n) {val -= n; return *this;}
+  IntIter& operator-=(difference_type n) noexcept {val -= n; return *this;}
 
   /**
    * The offset dereferencing operator does not throw.  No locking is
@@ -1603,9 +1586,9 @@ public:
    * require synchronization).
    * @return The integer value at the given offset.
    *
-   * Since 2.0.27.
+   * Since 2.0.27 and 2.2.10.
    */
-  reference operator[](difference_type n) const {return val + n;}
+  reference operator[](difference_type n) const noexcept {return val + n;}
 
   /**
    * The dereferencing operator does not throw.  No locking is carried
@@ -1620,9 +1603,9 @@ public:
    * require synchronization).
    * @return The integer value currently represented by the iterator.
    *
-   * Since 2.0.27.
+   * Since 2.0.27 and 2.2.10.
    */
-  reference operator*() const {return val;}
+  reference operator*() const noexcept {return val;}
 
 /* Only has effect if --with-glib-memory-slices-compat or
  * --with-glib-memory-slices-no-compat option picked */
@@ -1643,9 +1626,9 @@ public:
  * use of an IntIter object only by one of those functions does not
  * require synchronization).
  *
- * Since 2.0.27.
+ * Since 2.0.27 and 2.2.10.
  */
-inline bool operator==(IntIter iter1, IntIter iter2) {
+inline bool operator==(IntIter iter1, IntIter iter2) noexcept {
   return *iter1 == *iter2;
 }
 
@@ -1663,9 +1646,9 @@ inline bool operator==(IntIter iter1, IntIter iter2) {
  * use of an IntIter object only by one of those functions does not
  * require synchronization).
  *
- * Since 2.0.27.
+ * Since 2.0.27 and 2.2.10.
  */
-inline bool operator!=(IntIter iter1, IntIter iter2) {
+inline bool operator!=(IntIter iter1, IntIter iter2) noexcept {
   return !(iter1 == iter2);
 }
 
@@ -1683,9 +1666,9 @@ inline bool operator!=(IntIter iter1, IntIter iter2) {
  * use of an IntIter object only by one of those functions does not
  * require synchronization).
  *
- * Since 2.0.27.
+ * Since 2.0.27 and 2.2.10.
  */
-inline bool operator<(IntIter iter1, IntIter iter2) {
+inline bool operator<(IntIter iter1, IntIter iter2) noexcept {
   return *iter1 < *iter2;
 }
 
@@ -1703,9 +1686,9 @@ inline bool operator<(IntIter iter1, IntIter iter2) {
  * use of an IntIter object only by one of those functions does not
  * require synchronization).
  *
- * Since 2.0.27.
+ * Since 2.0.27 and 2.2.10.
  */
-inline bool operator>(IntIter iter1, IntIter iter2) {
+inline bool operator>(IntIter iter1, IntIter iter2) noexcept {
   return iter2 < iter1;
 }
 
@@ -1723,9 +1706,9 @@ inline bool operator>(IntIter iter1, IntIter iter2) {
  * use of an IntIter object only by one of those functions does not
  * require synchronization).
  *
- * Since 2.0.27.
+ * Since 2.0.27 and 2.2.10.
  */
-inline bool operator<=(IntIter iter1, IntIter iter2) {
+inline bool operator<=(IntIter iter1, IntIter iter2) noexcept {
   return !(iter1 > iter2);
 }
 
@@ -1743,9 +1726,9 @@ inline bool operator<=(IntIter iter1, IntIter iter2) {
  * use of an IntIter object only by one of those functions does not
  * require synchronization).
  *
- * Since 2.0.27.
+ * Since 2.0.27 and 2.2.10.
  */
-inline bool operator>=(IntIter iter1, IntIter iter2) {
+inline bool operator>=(IntIter iter1, IntIter iter2) noexcept {
   return !(iter1 < iter2);
 }
 
@@ -1763,9 +1746,9 @@ inline bool operator>=(IntIter iter1, IntIter iter2) {
  * use of an IntIter object only by one of those functions does not
  * require synchronization).
  *
- * Since 2.0.27.
+ * Since 2.0.27 and 2.2.10.
  */
-inline IntIter::difference_type operator-(IntIter iter1, IntIter iter2) {
+inline IntIter::difference_type operator-(IntIter iter1, IntIter iter2) noexcept {
   return *iter1 - *iter2;
 }
 
@@ -1783,9 +1766,9 @@ inline IntIter::difference_type operator-(IntIter iter1, IntIter iter2) {
  * use of an IntIter object only by one of those functions does not
  * require synchronization).
  *
- * Since 2.0.27.
+ * Since 2.0.27 and 2.2.10.
  */
-inline IntIter operator+(IntIter iter, IntIter::difference_type n) {
+inline IntIter operator+(IntIter iter, IntIter::difference_type n) noexcept {
   return IntIter{*iter + n};
 }
 
@@ -1803,9 +1786,9 @@ inline IntIter operator+(IntIter iter, IntIter::difference_type n) {
  * use of an IntIter object only by one of those functions does not
  * require synchronization).
  *
- * Since 2.0.27.
+ * Since 2.0.27 and 2.2.10.
  */
-inline IntIter operator-(IntIter iter, IntIter::difference_type n) {
+inline IntIter operator-(IntIter iter, IntIter::difference_type n) noexcept {
   return IntIter{*iter - n};
 }
 
@@ -1823,9 +1806,9 @@ inline IntIter operator-(IntIter iter, IntIter::difference_type n) {
  * use of an IntIter object only by one of those functions does not
  * require synchronization).
  *
- * Since 2.0.27.
+ * Since 2.0.27 and 2.2.10.
  */
-inline IntIter operator+(IntIter::difference_type n, IntIter iter) {
+inline IntIter operator+(IntIter::difference_type n, IntIter iter) noexcept {
   return iter + n;
 }
 
