@@ -1,4 +1,4 @@
-/* Copyright (C) 2004 to 2019 Chris Vine
+/* Copyright (C) 2004 to 2020 Chris Vine
 
 The library comprised in this file or of which this file is part is
 distributed by Chris Vine under the GNU Lesser General Public
@@ -249,7 +249,7 @@ public:
  */
 class CFree {
 public:
-  void operator()(const void* obj) {
+  void operator()(const void* obj) noexcept {
     std::free(const_cast<void*>(obj));
   }
 };
@@ -261,7 +261,7 @@ public:
  * template classes, which calls glib's g_free().
  * @ingroup handles
  * @details This functor enables those classes to manage memory
- * allocated by glib or gtk+ functions which requires to be freed with
+ * allocated by glib or gtk functions which requires to be freed with
  * g_free().  It is used in the typedefs @ref GcharSharedHandleAnchor
  * "GcharSharedHandle" and @ref GcharScopedHandleAnchor
  * "GcharScopedHandle".  It can also be used as the second template
@@ -269,7 +269,7 @@ public:
  */
 class GFree {
 public:
-  void operator()(const void* obj) {
+  void operator()(const void* obj) noexcept {
     g_free(const_cast<void*>(obj));
   }
 };
@@ -312,7 +312,7 @@ public:
  */
 template <class T> class GSliceFree {
 public:
-  void operator()(T obj) {
+  void operator()(T obj) noexcept {
     g_slice_free1(sizeof(*obj), (void*)obj);
   }
 };
@@ -362,7 +362,7 @@ public:
  *   SharedHandle<MyClass*, GSliceDestroy<MyClass*> > h; // won't throw
  *   { // scope block for p variable
  *     MyClass* p = g_slice_new(MyClass);
- *     try {new(p) MyClass;}                             // MyClass constructor might throw
+ *     try {p = new(p) MyClass;}                         // MyClass constructor might throw
  *     catch(...) {
  *       g_slice_free(MyClass, p);
  *       throw;
@@ -420,7 +420,7 @@ public:
  */
 template <gsize block_size> class GSliceFreeSize {
 public:
-  void operator()(const void* obj) {
+  void operator()(const void* obj) noexcept {
     g_slice_free1(block_size, const_cast<void*>(obj));
   }
 };
@@ -565,7 +565,7 @@ template <class T, class Dealloc = StandardArrayDelete<T>> class SharedHandle {
     }
   }
 
-  void reference() {
+  void reference() noexcept {
     if (!ref_items.ref_count_p) return;
     ++(*ref_items.ref_count_p);
   }
@@ -727,7 +727,7 @@ public:
   * The copy constructor does not throw.
   * @param sh_hand The handle to be copied.
   */
-  SharedHandle(const SharedHandle& sh_hand) {
+  SharedHandle(const SharedHandle& sh_hand) noexcept {
     ref_items = sh_hand.ref_items;
     reference();
   }
@@ -736,7 +736,7 @@ public:
   * The move constructor does not throw.  It has move semantics.
   * @param sh_hand The handle to be moved.
   */
-  SharedHandle(SharedHandle&& sh_hand) {
+  SharedHandle(SharedHandle&& sh_hand) noexcept {
     ref_items = sh_hand.ref_items;
     sh_hand.ref_items.ref_count_p = 0;
     sh_hand.ref_items.obj = 0;
@@ -762,21 +762,21 @@ public:
   * @return A pointer to the handled object (or NULL if none is
   * handled).
   */
-  T get() const {return ref_items.obj;}
+  T get() const noexcept {return ref_items.obj;}
 
  /**
   * This method does not throw.
   * @return A pointer to the handled object (or NULL if none is
   * handled).
   */
-  operator T() const {return ref_items.obj;}
+  operator T() const noexcept {return ref_items.obj;}
 
  /**
   * This method does not throw.
   * @return The number of SharedHandle objects referencing the managed
   * object (or 0 if none is managed by this SharedHandle).
   */
-  unsigned int get_refcount() const {return (ref_items.ref_count_p) ? *ref_items.ref_count_p : 0;}
+  unsigned int get_refcount() const noexcept {return (ref_items.ref_count_p) ? *ref_items.ref_count_p : 0;}
 
  /**
   * The destructor does not throw unless the destructor of a handled
@@ -820,7 +820,7 @@ public:
  * functions (such as in glib), because it provides a type conversion
  * operator.
  *
- * From version 2.0.19, this class has a move constructor and move
+ * From version 2.2.2, this class has a move constructor and move
  * assignment operator.  Prior to that, it could not be moved from or
  * to.
  */
@@ -844,9 +844,9 @@ public:
  * The move constructor does not throw.
  * @param sc_hand The handle to be moved.
  *
- * Since 2.0.19
+ * Since 2.0.19/2.2.2
  */
-  ScopedHandle(ScopedHandle&& sc_hand) {
+  ScopedHandle(ScopedHandle&& sc_hand) noexcept {
     obj = sc_hand.obj;
     sc_hand.obj = 0;
   }
@@ -858,7 +858,7 @@ public:
  * @param sc_hand The handle to be moved.
  * @return The ScopedHandle object after move assignment.
  *
- * Since 2.0.19
+ * Since 2.0.19/2.2.2
  */
   ScopedHandle& operator=(ScopedHandle&& sc_hand) {
     reset(sc_hand.release());
@@ -874,7 +874,7 @@ public:
  * objects (such as SharedHandle<const char*>), provided the deleter
  * functor will take such pointers.
  */
-  explicit ScopedHandle(T ptr = 0): obj(ptr) {}
+  explicit ScopedHandle(T ptr = 0) noexcept: obj(ptr) {}
 
 /**
  * Causes the ScopedHandle to delete its managed object (if any), and
@@ -895,21 +895,21 @@ public:
  * @return A pointer to the previously handled object (or NULL if none
  * was handled).
  */
-  T release() {T tmp = obj; obj = 0; return tmp;}
+  T release() noexcept {T tmp = obj; obj = 0; return tmp;}
 
 /**
  * This method does not throw.
  * @return A pointer to the handled object (or NULL if none is
  * handled).
  */
-  T get() const {return obj;}
+  T get() const noexcept {return obj;}
 
 /**
  * This method does not throw.
  * @return A pointer to the handled object (or NULL if none is
  * handled).
  */
-  operator T() const {return obj;}
+  operator T() const noexcept {return obj;}
 
 /**
  * The destructor does not throw unless the destructor of a handled
@@ -1109,7 +1109,7 @@ template <class T, class Dealloc = StandardArrayDelete<T>> class SharedLockHandl
   
   // SharedLockHandle<T, Dealloc>::reference() does not throw exceptions because
   // Thread::Mutex::Lock::Lock() and Thread::Mutex::Lock::~Lock() do not throw
-  void reference() {
+  void reference() noexcept {
     // we can (and should) check whether ref_items.ref_count_p is NULL without
     // a lock, because that member is specific to this SharedLockHandle object.
     // Only the integer pointed to by it is shared amongst SharedLockHandle
@@ -1389,7 +1389,7 @@ public:
   * The copy constructor does not throw.
   * @param sh_hand The handle to be copied.
   */
-  SharedLockHandle(const SharedLockHandle& sh_hand) {
+  SharedLockHandle(const SharedLockHandle& sh_hand) noexcept {
     ref_items = sh_hand.ref_items;
     reference();
   }
@@ -1398,7 +1398,7 @@ public:
   * The move constructor does not throw.  It has move semantics.
   * @param sh_hand The handle to be moved.
   */
-  SharedLockHandle(SharedLockHandle&& sh_hand) {
+  SharedLockHandle(SharedLockHandle&& sh_hand) noexcept {
     ref_items = sh_hand.ref_items;
 #ifdef CGU_SHARED_LOCK_HANDLE_USE_MUTEX
     sh_hand.ref_items.mutex_p = 0;  // make sure the value is valid as we may assign it
@@ -1426,14 +1426,14 @@ public:
   * @return A pointer to the handled object (or NULL if none is
   * handled).
   */
-  T get() const {return ref_items.obj;}
+  T get() const noexcept {return ref_items.obj;}
 
  /**
   * This method does not throw.
   * @return A pointer to the handled object (or NULL if none is
   * handled).
   */
-  operator T() const {return ref_items.obj;}
+  operator T() const noexcept {return ref_items.obj;}
 
  /**
   * This method does not throw.
@@ -1444,7 +1444,7 @@ public:
   * method is acted on.  It is provided as a utility, but may not be
   * meaningful, depending on the intended usage.
   */
-  unsigned int get_refcount() const {
+  unsigned int get_refcount() const noexcept {
     if (!ref_items.ref_count_p) return 0;
 #ifdef CGU_SHARED_LOCK_HANDLE_USE_MUTEX
     Thread::Mutex::Lock lock(*ref_items.mutex_p);
@@ -1474,7 +1474,7 @@ public:
  * Since 2.0.0-rc2
  */
 template <class T, class Dealloc>
-bool operator==(const SharedHandle<T, Dealloc>& s1, const SharedHandle<T, Dealloc>& s2) {
+bool operator==(const SharedHandle<T, Dealloc>& s1, const SharedHandle<T, Dealloc>& s2) noexcept {
   return (s1.get() == s2.get());
 }
 
@@ -1487,7 +1487,7 @@ bool operator==(const SharedHandle<T, Dealloc>& s1, const SharedHandle<T, Deallo
  * Since 2.0.0-rc2
  */
 template <class T, class Dealloc>
-bool operator!=(const SharedHandle<T, Dealloc>& s1, const SharedHandle<T, Dealloc>& s2) {
+bool operator!=(const SharedHandle<T, Dealloc>& s1, const SharedHandle<T, Dealloc>& s2) noexcept {
   return !(s1 == s2);
 }
 
@@ -1519,7 +1519,7 @@ bool operator<(const SharedHandle<T, Dealloc>& s1, const SharedHandle<T, Dealloc
  * Since 2.0.0-rc2
  */
 template <class T, class Dealloc>
-bool operator==(const SharedLockHandle<T, Dealloc>& s1, const SharedLockHandle<T, Dealloc>& s2) {
+bool operator==(const SharedLockHandle<T, Dealloc>& s1, const SharedLockHandle<T, Dealloc>& s2) noexcept {
   return (s1.get() == s2.get());
 }
 
@@ -1532,7 +1532,7 @@ bool operator==(const SharedLockHandle<T, Dealloc>& s1, const SharedLockHandle<T
  * Since 2.0.0-rc2
  */
 template <class T, class Dealloc>
-bool operator!=(const SharedLockHandle<T, Dealloc>& s1, const SharedLockHandle<T, Dealloc>& s2) {
+bool operator!=(const SharedLockHandle<T, Dealloc>& s1, const SharedLockHandle<T, Dealloc>& s2) noexcept {
   return !(s1 == s2);
 }
 
