@@ -194,12 +194,10 @@ TaskManager::TaskManager(unsigned int max, unsigned int min,
   for (unsigned int count = 0; count < min; ++count) {
     std::unique_ptr<Thread> t;
     try {
-      // the following formula will work for both std::auto_ptr and std::unique_ptr
-      // return values of Cgu::Thread::Thread::start()
-      t.reset(Thread::start(Callback::make(*ref_impl,
-                                           &TaskManager::RefImpl::do_tasks,
-                                           true),
-                            false).release());
+      t = Thread::start(Callback::make(*ref_impl,
+				       &TaskManager::RefImpl::do_tasks,
+				       true),
+			false);
     }
     catch (...) {
       ref_impl->stop_status = StopStatus::stopped;
@@ -294,12 +292,10 @@ void TaskManager::set_max_threads_impl(unsigned int max, Mutex::TrackLock& lock)
     for (; new_thread_count; --new_thread_count) {
       std::unique_ptr<Thread> t;
       try {
-	// the following formula will work for both std::auto_ptr and
-	// std::unique_ptr return values of Cgu::Thread::Thread::start()
-	t.reset(Thread::start(Callback::make(*ref_impl,
-					     &TaskManager::RefImpl::do_tasks,
-					     false),
-			      false).release());
+	t = Thread::start(Callback::make(*ref_impl,
+					 &TaskManager::RefImpl::do_tasks,
+					 false),
+			  false);
       }
       catch (...) { // try block might throw std::bad_alloc
 	// roll back for any unstarted threads
@@ -409,8 +405,8 @@ void TaskManager::stop_all() {
       catch (...) { // if the push throws, it is guaranteed that the
                     // item concerned is not inserted in the queue,
                     // because std::unique_ptr's move constructor and
-                    // move assignment operator do not throw, so we can
-                    // rethrow here without issues
+                    // move assignment operator do not throw, so we
+                    // can rethrow here without issues
 	ref_impl->error = true;
 	throw;
       }
@@ -428,19 +424,12 @@ void TaskManager::stop_all() {
     while (tmp->used_threads) tmp->cond.wait(tmp->mutex);
     lock.unlock(); // we cannot do a final unreference of RefImpl
 		   // still holding its mutex
-    tmp->unref();
+    tmp->unref(); 
   }
 }
 
-void TaskManager::add_task(
-#ifdef CGU_USE_AUTO_PTR
-			   std::auto_ptr<const Callback::Callback> task,
-			   std::auto_ptr<const Callback::Callback> fail
-#else
-			   std::unique_ptr<const Callback::Callback> task,
-			   std::unique_ptr<const Callback::Callback> fail
-#endif
-			   ) {
+void TaskManager::add_task(std::unique_ptr<const Callback::Callback> task,
+			   std::unique_ptr<const Callback::Callback> fail) {
 
 
   { // scope block for mutex lock
@@ -474,12 +463,10 @@ void TaskManager::add_task(
 
       std::unique_ptr<Thread> t;
       try {
-	// the following formula will work for both std::auto_ptr and std::unique_ptr
-	// return values of Cgu::Thread::Thread::start()
-	t.reset(Thread::start(Callback::make(*ref_impl,
-					     &TaskManager::RefImpl::do_tasks,
-					     false),
-			      false).release());
+	t = Thread::start(Callback::make(*ref_impl,
+					 &TaskManager::RefImpl::do_tasks,
+					 false),
+			  false);
       }
       catch (...) { // try block might throw std::bad_alloc
 	// roll back for the unstarted thread
@@ -529,12 +516,7 @@ void TaskManager::add_task(
   Mutex::TrackLock stop_lock{ref_impl->stop_guard};
   if (ref_impl->stop_status == StopStatus::running) {
     try {
-      // the following formula will work for both std::auto_ptr and std::unique_ptr
-      // argument values of this method
-      ref_impl->task_queue.emplace(
-	std::unique_ptr<const Callback::Callback>(task.release()),
-        std::unique_ptr<const Callback::Callback>(fail.release())
-      );
+      ref_impl->task_queue.emplace(std::move(task), std::move(fail));
     }
     catch (...) {
       // roll back for the unadded task
