@@ -1,4 +1,4 @@
-/* Copyright (C) 2007, 2011, 2013 and 2017 to 2019 Chris Vine
+/* Copyright (C) 2007, 2011, 2013 and 2017 to 2020 Chris Vine
 
 The library comprised in this file or of which this file is part is
 distributed by Chris Vine under the GNU Lesser General Public
@@ -611,7 +611,10 @@ GObject* TextPrintManager::create_custom_widget_impl(GtkPrintOperation* print_op
   GtkGrid* grid_p = (GtkGrid*)gtk_grid_new();
   g_object_set(grid_p,
 	       "valign", GTK_ALIGN_CENTER,
-	       "margin", 12,
+	       "margin-top", 15,
+	       "margin-bottom", 15,
+	       "margin-start", 15,
+	       "margin-end", 15,
 	       static_cast<void*>(0));
   gtk_grid_attach(grid_p, font_label_p, 0, 0, 1, 1);
   gtk_grid_attach(grid_p, font_entry_h, 1, 0, 1, 1);
