@@ -1,4 +1,4 @@
-/* Copyright (C) 2012, 2017 and 2018 Chris Vine
+/* Copyright (C) 2012, 2017, 2018 and 2020 Chris Vine
 
 The library comprised in this file or of which this file is part is
 distributed by Chris Vine under the GNU Lesser General Public
@@ -55,24 +55,29 @@ void message_button_clicked(GtkWidget* w, void* data) {
 }
 
 ProgWin::ProgWin(): WinBase{"Print test", 0, false} {
-#if GTK_CHECK_VERSION(3,96,0)
+#if GTK_CHECK_VERSION(3,98,4)
   GtkWidget* box = gtk_box_new(GTK_ORIENTATION_VERTICAL, 2);
-  g_object_set(box, "margin", 5, static_cast<void*>(0));
-  gtk_container_add(GTK_CONTAINER(get_win()), box);
+  g_object_set(box,
+	       "margin-bottom", 5,
+	       "margin-top", 5,
+	       "margin-start", 5,
+	       "margin-end", 5,
+	       static_cast<void*>(0));
+  gtk_window_set_child(get_win(), box);
   GtkWidget* label = gtk_label_new("Print test page?");
   gtk_widget_set_vexpand(label, true);
   gtk_widget_set_valign(label, GTK_ALIGN_CENTER);
-  gtk_container_add(GTK_CONTAINER(box), label);
+  gtk_box_append(GTK_BOX(box), label);
   GtkWidget* button_box = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 5);
   gtk_box_set_homogeneous(GTK_BOX(button_box), true);
-  gtk_container_add(GTK_CONTAINER(box), button_box);
-  gtk_widget_set_valign(label, GTK_ALIGN_CENTER);
+  gtk_widget_set_valign(button_box, GTK_ALIGN_CENTER);
+  gtk_box_append(GTK_BOX(box), button_box);
   close_button = gtk_button_new_with_label("Close");
-  gtk_container_add(GTK_CONTAINER(button_box), close_button);
+  gtk_box_append(GTK_BOX(button_box), close_button);
   g_signal_connect(G_OBJECT(close_button), "clicked",
                    G_CALLBACK(message_button_clicked), this);
   ok_button = gtk_button_new_with_label("OK");
-  gtk_container_add(GTK_CONTAINER(button_box), ok_button);
+  gtk_box_append(GTK_BOX(button_box), ok_button);
   g_signal_connect(G_OBJECT(ok_button), "clicked",
                    G_CALLBACK(message_button_clicked), this);
 #else
