@@ -248,6 +248,86 @@ static void test_iowatch_tracked() {
 
   g_main_loop_unref(loop);
 }
+
+static void test_iowatch_lambda() {
+
+  loop = g_main_loop_new(0, false);
+
+  test = new Test;
+  result.clear();
+  {
+    PipeFifo tmp{PipeFifo::block};
+    io_pipe = std::move(tmp);
+  }
+  start_iowatch(io_pipe.get_read_fd(),
+		[] (GIOCondition c, bool& keep) {int max = 2; test->iowatch_untracked_cond_func(max, c, keep);},
+                GIOCondition(G_IO_IN | G_IO_HUP | G_IO_ERR));
+  Thread::JoinableHandle t1{Thread::Thread::start(Callback::make(thread_func), true),
+                            Thread::JoinableHandle::join_on_exit};
+  start_timeout(200, Callback::make(&timeout_func, 2), G_PRIORITY_LOW);
+  g_main_loop_run(loop);
+  t1.join();
+  delete test;
+  g_assert_cmpuint(result.size(), ==, 2);
+  g_assert_cmpuint(result[0], ==, (unsigned int)'a');
+  g_assert_cmpuint(result[1], ==, (unsigned int)'b');
+
+  test = new Test;
+  result.clear();
+  {
+    PipeFifo tmp{PipeFifo::block};
+    io_pipe = std::move(tmp);
+  }
+
+  test = new Test;
+  result.clear();
+  {
+    PipeFifo tmp{PipeFifo::block};
+    io_pipe = std::move(tmp);
+  }
+  start_iowatch(io_pipe.get_read_fd(),
+		[] (GIOCondition c, bool& keep) {int max = 2; test->iowatch_untracked_cond_func(max, c, keep);},
+                test->releaser, // not used
+                GIOCondition(G_IO_IN | G_IO_HUP | G_IO_ERR));
+  Thread::JoinableHandle t2{Thread::Thread::start(Callback::make(thread_func), true),
+                            Thread::JoinableHandle::join_on_exit};
+  start_timeout(200, Callback::make(&timeout_func, 2), G_PRIORITY_LOW);
+  g_main_loop_run(loop);
+  t2.join();
+  delete test;
+  g_assert_cmpuint(result.size(), ==, 2);
+  g_assert_cmpuint(result[0], ==, (unsigned int)'a');
+  g_assert_cmpuint(result[1], ==, (unsigned int)'b');
+
+  test = new Test;
+  result.clear();
+  {
+    PipeFifo tmp{PipeFifo::block};
+    io_pipe = std::move(tmp);
+  }
+
+  test = new Test;
+  result.clear();
+  {
+    PipeFifo tmp{PipeFifo::block};
+    io_pipe = std::move(tmp);
+  }
+  start_iowatch(io_pipe.get_read_fd(),
+		[] (GIOCondition c, bool& keep) {int max = 3; test->iowatch_tracked_cond_func(max, c, keep);},
+		test->releaser,
+                GIOCondition(G_IO_IN | G_IO_HUP | G_IO_ERR));
+  Thread::JoinableHandle t3{Thread::Thread::start(Callback::make(thread_func), true),
+                            Thread::JoinableHandle::join_on_exit};
+  start_timeout(200, Callback::make(&timeout_func, 3), G_PRIORITY_LOW);
+  g_main_loop_run(loop);
+  t3.join();
+  g_assert_cmpuint(result.size(), ==, 3);
+  g_assert_cmpuint(result[0], ==, (unsigned int)'a');
+  g_assert_cmpuint(result[1], ==, (unsigned int)'b');
+  g_assert_cmpuint(result[2], ==, (unsigned int)'c');
+
+  g_main_loop_unref(loop);
+}
 } // extern "C"
 
 
@@ -256,6 +336,7 @@ int main (int argc, char* argv[]) {
 
   g_test_add_func("/iowatch/iowatch_untracked", test_iowatch_untracked);
   g_test_add_func("/iowatch/iowatch_tracked", test_iowatch_tracked);
+  g_test_add_func("/iowatch/iowatch_lambda", test_iowatch_lambda);
 
   return g_test_run();
 }
