@@ -23,6 +23,9 @@ License as follows:
 
 #include <exception>
 #include <functional>
+#include <memory>
+#include <future>
+#include <string>
 
 #include <glib.h>
 
@@ -158,20 +161,10 @@ void no_when(const int&) {
   g_assert_not_reached();
 }
 
-void upause_and_quit(GMainLoop* loop, int microsec) {
-  g_usleep(microsec);
-  Callback::post(Callback::make(&g_main_loop_quit, loop));
-}
-
 void no_fail() {
   g_assert_not_reached();
 }
 
-int upause(int microsec) {
-  g_usleep(microsec); 
-  return 0;
-}
-
 /***************** end of test support stuff *****************/
 
 extern "C" {
@@ -182,6 +175,7 @@ static void test_used_threads() {
     mutex.unlock();
     Thread::TaskManager tm;
 
+    // use deprecated add_task() functions
     tm.add_task(Callback::make(&task1, 1000));
     tm.add_task(Callback::make(&task1, 1000));
     tm.add_task(Callback::make(&task1, 1000));
@@ -209,10 +203,10 @@ static void test_used_threads() {
     mutex.unlock();
     Thread::TaskManager tm(3, 2, 10000);
 
-    tm.add_task(Callback::make(&task2, 1000));
-    tm.add_task(Callback::make(&task2, 1000));
-    tm.add_task(Callback::make(&task2, 1000));
-    tm.add_task(Callback::make(&task2, 1000));
+    tm.add_task([] () {task2(1000);});
+    tm.add_task([] () {task2(1000);});
+    tm.add_task([] () {task2(1000);});
+    tm.add_task([] () {task2(1000);});
     mutex.lock();
     timespec ts;
     Thread::Cond::get_abs_time(ts, 5000);
@@ -348,13 +342,8 @@ static void test_fail() {
     mutex.unlock();
     Thread::TaskManager tm;
 
-#ifdef CGU_USE_AUTO_PTR
-    tm.add_task(std::auto_ptr<const Callback::Callback>(Callback::make(&task3)),
-		std::auto_ptr<const Callback::Callback>(Callback::make(&fail_cb)));
-#else
     tm.add_task(to_unique(Callback::make(&task3)),
 		to_unique(Callback::make(&fail_cb)));
-#endif
     mutex.lock();
     timespec ts;
     Thread::Cond::get_abs_time(ts, 5000);
@@ -370,13 +359,43 @@ static void test_fail() {
     mutex.unlock();
     Thread::TaskManager tm;
 
-#ifdef CGU_USE_AUTO_PTR
-    tm.add_task(std::auto_ptr<const Callback::Callback>(Callback::make(&task4)),
-		std::auto_ptr<const Callback::Callback>(Callback::make(&fail_cb)));
-#else
     tm.add_task(to_unique(Callback::make(&task4)),
 		to_unique(Callback::make(&fail_cb)));
-#endif
+    mutex.lock();
+    timespec ts;
+    Thread::Cond::get_abs_time(ts, 5000);
+    while (!count) {
+      if (cond.timed_wait(mutex, ts)) g_assert_not_reached();
+    }
+    mutex.unlock();
+  }  
+
+  {
+    mutex.lock();
+    count = 0;
+    mutex.unlock();
+    Thread::TaskManager tm;
+
+    tm.add_task([] () {task3();},
+		[] () {fail_cb();});
+    mutex.lock();
+    timespec ts;
+    Thread::Cond::get_abs_time(ts, 5000);
+    while (!count) {
+      if (cond.timed_wait(mutex, ts)) g_assert_not_reached();
+    }
+    mutex.unlock();
+  }  
+
+  {
+    mutex.lock();
+    count = 0;
+    mutex.unlock();
+    Thread::TaskManager tm;
+
+    auto l1 = [] () {task4();};
+    auto l2 = [] () {fail_cb();};
+    tm.add_task(l1, l2);
     mutex.lock();
     timespec ts;
     Thread::Cond::get_abs_time(ts, 5000);
@@ -419,7 +438,7 @@ static void test_task_result() {
     auto res2 = tm.make_task_result(t, &Test::add2, one, 2);
     auto res3 = tm.make_task_result(t, &Test::add3, one, 2, 3);
     auto res4 = tm.make_task_result(t, &Test::add_fail1);
-    auto res5 = tm.make_task_result(t, &Test::add_fail2);
+    auto res5 = tm.make_task_result([&t] () mutable {return t.add_fail2();});
     g_assert_cmpint(res0->get(), ==, 1);
     g_assert_cmpint(res1->get(), ==, 2);
     g_assert_cmpint(res2->get(), ==, 4);
@@ -439,7 +458,7 @@ static void test_task_result() {
     auto res1 = tm.make_task_result(t, &ConstTest::add1, one);
     auto res2 = tm.make_task_result(t, &ConstTest::add2, one, 2);
     auto res3 = tm.make_task_result(t, &ConstTest::add3, one, 2, 3);
-    auto res4 = tm.make_task_result(t, &ConstTest::add_fail1);
+    auto res4 = tm.make_task_result([&t] () {return t.add_fail1();});
     auto res5 = tm.make_task_result(t, &ConstTest::add_fail2);
     g_assert_cmpint(res0->get(), ==, 1);
     g_assert_cmpint(res1->get(), ==, 2);
@@ -493,13 +512,37 @@ static void test_task_result() {
   }
 }
 
-static void test_task_when() {
+static void test_task_packaged() {
 
-#ifdef CGU_USE_AUTO_PTR
-  typedef std::auto_ptr<const Callback::CallbackArg<const int&>> WhenCbPtr;
-#else
-  typedef std::unique_ptr<const Callback::CallbackArg<const int&>> WhenCbPtr;
-#endif
+  Thread::TaskManager tm;
+
+  // make some arguments temporaries and some named arguments
+  int four = 4;
+  int thrown = 0;
+  std::future<int> res1 = tm.make_task_packaged(std::bind(&add2, 3, four));
+  auto f1 = std::bind(&add3, 3, four, 5);
+  auto res2 = tm.make_task_packaged(f1);
+  auto res3 = tm.make_task_packaged(std::function<int()>(&add_fail1));
+  auto f2 = [] () {g_usleep(1000); add_fail2();};
+  auto res4 = tm.make_task_packaged(f2);
+  g_assert_cmpint(res1.get(), ==, 7);
+  g_assert_cmpint(res2.get(), ==, 12);
+  try {
+    res3.get();
+  }
+  catch (Thread::Exit&) {
+    ++thrown;
+  }
+  try {
+    res4.get();
+  }
+  catch (std::exception&) {
+    ++thrown;
+  }
+  g_assert_cmpint(thrown, ==, 2);
+}
+
+static void test_task_when() {
 
   Thread::TaskManager tm;
   GMainLoop* loop = g_main_loop_new(0, true);
@@ -512,14 +555,14 @@ static void test_task_when() {
 
     // make some arguments temporaries and some named arguments
     int one = 1;
-    tm.make_task_when(WhenCbPtr(Callback::make(&when, loop, 4U, 1)),
+    tm.make_task_when(to_unique(Callback::make(&when, loop, 4U, 1)),
 		      0, t, &Test::add0);
-    tm.make_task_when(WhenCbPtr(Callback::make(&when, loop, 4U, 2)),
+    tm.make_task_when(to_unique(Callback::make(&when, loop, 4U, 2)),
 		      0, t, &Test::add1, one);
-    tm.make_task_when(WhenCbPtr(Callback::make(&when, loop, 4U, 4)),
-		      0, t, &Test::add2, one, 2);
-    tm.make_task_when(WhenCbPtr(Callback::make(&when, loop, 4U, 7)),
-		      0, t, &Test::add3, one, 2, 3);
+    tm.make_task_when(to_unique(Callback::make(&when, loop, 4U, 4)),
+		      0, [&t, one] () -> int {return t.add2(one, 2);});
+    tm.make_task_when([loop] (const int& i) {when(loop, 4U, 7, i);},
+		      0, [&t, one] () -> int {return t.add3(one, 2, 3);});
     g_main_loop_run(loop);
     mutex.lock();
     g_assert_cmpuint(count, ==, 4U);
@@ -534,14 +577,14 @@ static void test_task_when() {
 
     // make some arguments temporaries and some named arguments
     int one = 1;
-    tm.make_task_when(WhenCbPtr(Callback::make(&when, loop, 4U, 1)),
+    tm.make_task_when(to_unique(Callback::make(&when, loop, 4U, 1)),
 		      0, t, &ConstTest::add0);
-    tm.make_task_when(WhenCbPtr(Callback::make(&when, loop, 4U, 2)),
+    tm.make_task_when(to_unique(Callback::make(&when, loop, 4U, 2)),
 		      0, t, &ConstTest::add1, one);
-    tm.make_task_when(WhenCbPtr(Callback::make(&when, loop, 4U, 4)),
-		      0, t, &ConstTest::add2, one, 2);
-    tm.make_task_when(WhenCbPtr(Callback::make(&when, loop, 4U, 7)),
-		      0, t, &ConstTest::add3, one, 2, 3);
+    tm.make_task_when(to_unique(Callback::make(&when, loop, 4U, 4)),
+		      0, [&t, one] () -> int {return t.add2(one, 2);});
+    tm.make_task_when([loop] (const int& i) {when(loop, 4U, 7, i);},
+		      0, [&t, one] () -> int {return t.add3(one, 2, 3);});
     g_main_loop_run(loop);
     mutex.lock();
     g_assert_cmpuint(count, ==, 4U);
@@ -555,15 +598,15 @@ static void test_task_when() {
 
     // make some arguments temporaries and some named arguments
     int one = 1, four = 4;
-    tm.make_task_when(WhenCbPtr(Callback::make(&when, loop, 5U, 0)),
+    tm.make_task_when(to_unique(Callback::make(&when, loop, 5U, 0)),
 		      0, &add0);
-    tm.make_task_when(WhenCbPtr(Callback::make(&when, loop, 5U, 1)),
-		      0, &add1, one);
-    tm.make_task_when(WhenCbPtr(Callback::make(&when, loop, 5U, 3)),
-		      0, &add2, one, 2);
-    tm.make_task_when(WhenCbPtr(Callback::make(&when, loop, 5U, 6)),
-		      0, &add3, one, 2, 3);
-    tm.make_task_when(WhenCbPtr(Callback::make(&when, loop, 5U, 10)),
+    tm.make_task_when(to_unique(Callback::make(&when, loop, 5U, 1)),
+		      0, [one] () -> int {return add1(one);});
+    tm.make_task_when([loop] (const int& i) {when(loop, 5U, 3, i);},
+		      0, [one] () -> int {return add2(one, 2);});
+    tm.make_task_when([loop] (const int& i) {return when(loop, 5U, 6, i);},
+		      0, [one] () -> int {return add3(one, 2, 3);});
+    tm.make_task_when(to_unique(Callback::make(&when, loop, 5U, 10)),
 		      0, &add4, one, 2, 3, four);
     g_main_loop_run(loop);
     mutex.lock();
@@ -578,10 +621,10 @@ static void test_task_when() {
 
     // make some arguments temporaries and some named arguments
     int four = 4;
-    tm.make_task_when<int>(WhenCbPtr(Callback::make(&when, loop, 2U, 7)),
+    tm.make_task_when<int>(to_unique(Callback::make(&when, loop, 2U, 7)),
 			   0, std::bind(&add2, 3, four));
     std::function<int()> f(std::bind(&add3, 3, four, 5));
-    tm.make_task_when(WhenCbPtr(Callback::make(&when, loop, 2U, 12)),
+    tm.make_task_when(to_unique(Callback::make(&when, loop, 2U, 12)),
 		      0, f);
     g_main_loop_run(loop);
     mutex.lock();
@@ -592,12 +635,6 @@ static void test_task_when() {
 
 static void test_task_compose() {
 
-#ifdef CGU_USE_AUTO_PTR
-  typedef std::auto_ptr<const Callback::CallbackArg<const int&>> WhenCbPtr;
-#else
-  typedef std::unique_ptr<const Callback::CallbackArg<const int&>> WhenCbPtr;
-#endif
-
   Thread::TaskManager tm;
   GMainLoop* loop = g_main_loop_new(0, true);
 
@@ -608,26 +645,20 @@ static void test_task_compose() {
   // make some arguments temporaries and some named arguments
   int four = 4;
   tm.make_task_compose<int>(std::bind(&add2, 3, four), 0,
-			    WhenCbPtr(Callback::make(&when, loop, 2U, 7)));
+			    to_unique(Callback::make(&when, loop, 3U, 7)));
   std::function<int()> f(std::bind(&add3, 3, four, 5));
   tm.make_task_compose(f, 0,
-		       WhenCbPtr(Callback::make(&when, loop, 2U, 12)));
+		       [loop] (const int& i) {when(loop, 3U, 12, i);});
+  tm.make_task_compose([four] () -> int {return add3(four, 2, 3);},
+		       0, [loop] (const int& i) {when(loop, 3U, 9, i);});
   g_main_loop_run(loop);
   mutex.lock();
-  g_assert_cmpuint(count, ==, 2U);
+  g_assert_cmpuint(count, ==, 3U);
   mutex.unlock();
 }
 
 static void test_task_when_full() {
 
-#ifdef CGU_USE_AUTO_PTR
-  typedef std::auto_ptr<const Callback::CallbackArg<const int&>> WhenCbPtr;
-  typedef std::auto_ptr<const Callback::Callback> FailCbPtr;
-#else
-  typedef std::unique_ptr<const Callback::CallbackArg<const int&>> WhenCbPtr;
-  typedef std::unique_ptr<const Callback::Callback> FailCbPtr;
-#endif
-
   // TaskManager::make_test_when_full() is of necessity tested by
   // test_task_when().  All we need to do here is test the fail
   // callback and releasers
@@ -641,12 +672,12 @@ static void test_task_when_full() {
     mutex.unlock();
     Test t;
 
-    tm.make_task_when_full(WhenCbPtr(Callback::make(&no_when)), 0,
-			   FailCbPtr(Callback::make(&fail, loop, 2U)),
+    tm.make_task_when_full(to_unique(Callback::make(&no_when)), 0,
+			   to_unique(Callback::make(&fail, loop, 2U)),
 			   &t.releaser,
 			   G_PRIORITY_DEFAULT, 0, t, &Test::add_fail1);
-    tm.make_task_when_full(WhenCbPtr(Callback::make(&no_when)), 0,
-			   FailCbPtr(Callback::make(&fail, loop, 2U)), 0,
+    tm.make_task_when_full(to_unique(Callback::make(&no_when)), 0,
+			   to_unique(Callback::make(&fail, loop, 2U)), 0,
 			   G_PRIORITY_DEFAULT, 0, t, &Test::add_fail2);
     g_main_loop_run(loop);
     mutex.lock();
@@ -660,11 +691,11 @@ static void test_task_when_full() {
     mutex.unlock();
     const ConstTest t;
 
-    tm.make_task_when_full(WhenCbPtr(Callback::make(&no_when)), 0,
-			   FailCbPtr(Callback::make(&fail, loop, 2U)), 0,
+    tm.make_task_when_full(to_unique(Callback::make(&no_when)), 0,
+			   to_unique(Callback::make(&fail, loop, 2U)), 0,
 			   G_PRIORITY_DEFAULT, 0, t, &ConstTest::add_fail1);
-    tm.make_task_when_full(WhenCbPtr(Callback::make(&no_when)), 0,
-			   FailCbPtr(Callback::make(&fail, loop, 2U)),
+    tm.make_task_when_full(to_unique(Callback::make(&no_when)), 0,
+			   to_unique(Callback::make(&fail, loop, 2U)),
 			   &t.releaser,
 			   G_PRIORITY_DEFAULT, 0, t, &ConstTest::add_fail2);
     g_main_loop_run(loop);
@@ -679,12 +710,12 @@ static void test_task_when_full() {
     mutex.unlock();
     Test t;
 
-    tm.make_task_when_full(WhenCbPtr(Callback::make(&no_when)), 0,
-			   FailCbPtr(Callback::make(&fail, loop, 2U)),
+    tm.make_task_when_full(to_unique(Callback::make(&no_when)), 0,
+			   to_unique(Callback::make(&fail, loop, 2U)),
 			   &t.releaser,
 			   G_PRIORITY_DEFAULT, 0, &add_fail1);
-    tm.make_task_when_full(WhenCbPtr(Callback::make(&no_when)), 0,
-			   FailCbPtr(Callback::make(&fail, loop, 2U)), 0,
+    tm.make_task_when_full(to_unique(Callback::make(&no_when)), 0,
+			   to_unique(Callback::make(&fail, loop, 2U)), 0,
 			   G_PRIORITY_DEFAULT, 0, &add_fail2);
     g_main_loop_run(loop);
     mutex.lock();
@@ -698,13 +729,13 @@ static void test_task_when_full() {
     mutex.unlock();
     Test t;
 
-    tm.make_task_when_full(WhenCbPtr(Callback::make(&no_when)), 0,
-			   FailCbPtr(Callback::make(&fail, loop, 2U)),
+    tm.make_task_when_full([] (const int& i) {no_when(i);}, 0,
+			   [loop] () {fail(loop, 2U);},
 			   &t.releaser,
-			   G_PRIORITY_DEFAULT, 0, std::bind(&Test::add_fail1, &t));
-    tm.make_task_when_full(WhenCbPtr(Callback::make(&no_when)), 0,
-			   FailCbPtr(Callback::make(&fail, loop, 2U)), 0,
-			   G_PRIORITY_DEFAULT, 0, std::bind(&Test::add_fail2, &t));
+			   G_PRIORITY_DEFAULT, 0, [&t]() {return t.add_fail1();});
+    tm.make_task_when_full([] (const int& i) {no_when(i);}, 0,
+			   [loop] () {fail(loop, 2U);}, 0,
+			   G_PRIORITY_DEFAULT, 0, [&t]() {return t.add_fail2();});
     g_main_loop_run(loop);
     mutex.lock();
     g_assert_cmpuint(count, ==, 2U);
@@ -718,27 +749,27 @@ static void test_task_when_full() {
     mutex.unlock();
     Test t;
     const ConstTest ct;
-    tm.make_task_when_full(WhenCbPtr(Callback::make(&when, loop, 4U, 2)),
+    tm.make_task_when_full(to_unique(Callback::make(&when, loop, 4U, 2)),
 			   &t.releaser,
-			   FailCbPtr(Callback::make(&no_fail)),
+			   to_unique(Callback::make(&no_fail)),
 			   &t.releaser,
 			   G_PRIORITY_DEFAULT, 0, t, &Test::add1, 1);
-    tm.make_task_when_full(WhenCbPtr(Callback::make(&when, loop, 4U, 2)),
+    tm.make_task_when_full(to_unique(Callback::make(&when, loop, 4U, 2)),
 			   &t.releaser,
-			   FailCbPtr(Callback::make(&no_fail)),
+			   to_unique(Callback::make(&no_fail)),
 			   &t.releaser,
 			   G_PRIORITY_DEFAULT, 0, ct, &ConstTest::add1, 1);
-    tm.make_task_when_full(WhenCbPtr(Callback::make(&when, loop, 4U, 1)),
+    tm.make_task_when_full(to_unique(Callback::make(&when, loop, 4U, 1)),
 			   &t.releaser,
-			   FailCbPtr(Callback::make(&no_fail)),
+			   to_unique(Callback::make(&no_fail)),
 			   &t.releaser,
 			   G_PRIORITY_DEFAULT, 0, &add1, 1);
-    tm.make_task_when_full(WhenCbPtr(Callback::make(&when, loop, 4U, 1)),
+    tm.make_task_when_full(to_unique(Callback::make(&when, loop, 4U, 1)),
 			   &t.releaser,
-			   FailCbPtr(Callback::make(&no_fail)),
+			   to_unique(Callback::make(&no_fail)),
 			   &t.releaser,
 			   G_PRIORITY_DEFAULT, 0,
-			   std::bind(&add1, 1));
+			   [] () {return add1(1);});
     g_main_loop_run(loop);
     mutex.lock();
     g_assert_cmpuint(count, ==, 4U);
@@ -750,64 +781,170 @@ static void test_task_when_full() {
     const ConstTest ct;
     {
       Releaser releaser;
-      tm.make_task_when_full(WhenCbPtr(Callback::make(&no_when)),
+      tm.make_task_when_full(to_unique(Callback::make(&no_when)),
 			     &releaser,
-			     FailCbPtr(Callback::make(&no_fail)),
+			     to_unique(Callback::make(&no_fail)),
 			     &releaser,
 			     G_PRIORITY_DEFAULT, 0,
 			     t, &Test::add0);
-      tm.make_task_when_full(WhenCbPtr(Callback::make(&no_when)),
+      tm.make_task_when_full(to_unique(Callback::make(&no_when)),
 			     &releaser,
-			     FailCbPtr(Callback::make(&no_fail)),
+			     to_unique(Callback::make(&no_fail)),
 			     &releaser,
 			     G_PRIORITY_DEFAULT, 0,
 			     t, &Test::add_fail1);
 
-      tm.make_task_when_full(WhenCbPtr(Callback::make(&no_when)),
+      tm.make_task_when_full(to_unique(Callback::make(&no_when)),
 			     &releaser,
-			     FailCbPtr(Callback::make(&no_fail)),
+			     to_unique(Callback::make(&no_fail)),
 			     &releaser,
 			     G_PRIORITY_DEFAULT, 0,
 			     ct, &ConstTest::add0);
-      tm.make_task_when_full(WhenCbPtr(Callback::make(&no_when)),
+      tm.make_task_when_full(to_unique(Callback::make(&no_when)),
 			     &releaser,
-			     FailCbPtr(Callback::make(&no_fail)),
+			     to_unique(Callback::make(&no_fail)),
 			     &releaser,
 			     G_PRIORITY_DEFAULT, 0,
 			     ct, &ConstTest::add_fail1);
 
-      tm.make_task_when_full(WhenCbPtr(Callback::make(&no_when)),
+      tm.make_task_when_full(to_unique(Callback::make(&no_when)),
 			     &releaser,
-			     FailCbPtr(Callback::make(&no_fail)),
+			     to_unique(Callback::make(&no_fail)),
 			     &releaser,
 			     G_PRIORITY_DEFAULT, 0,
 			     &add0);
-      tm.make_task_when_full(WhenCbPtr(Callback::make(&no_when)),
+      tm.make_task_when_full(to_unique(Callback::make(&no_when)),
 			     &releaser,
-			     FailCbPtr(Callback::make(&no_fail)),
+			     to_unique(Callback::make(&no_fail)),
 			     &releaser,
 			     G_PRIORITY_DEFAULT, 0,
 			     &add_fail1);
 
-      tm.make_task_when_full(WhenCbPtr(Callback::make(&no_when)),
+      tm.make_task_when_full(to_unique(Callback::make(&no_when)),
 			     &releaser,
-			     FailCbPtr(Callback::make(&no_fail)),
+			     to_unique(Callback::make(&no_fail)),
 			     &releaser,
 			     G_PRIORITY_DEFAULT, 0,
-			     std::bind(&Test::add0, &t));
-      tm.make_task_when_full(WhenCbPtr(Callback::make(&no_when)),
+			     [&t]() {return t.add0();});
+      tm.make_task_when_full(to_unique(Callback::make(&no_when)),
 			     &releaser,
-			     FailCbPtr(Callback::make(&no_fail)),
+			     to_unique(Callback::make(&no_fail)),
 			     &releaser,
 			     G_PRIORITY_DEFAULT, 0,
-			     std::bind(&Test::add_fail1, &t));
+			     [&t]() {return t.add_fail1();});
 
-      tm.add_task(Callback::make(&upause_and_quit, loop, 100000));
+      tm.add_task([loop]() -> void {
+	  g_usleep(100000);
+	  Callback::post([loop] () -> void {
+	      g_main_loop_quit(loop);
+	    });
+	});
     }
     g_main_loop_run(loop);
   }
 }
 
+static void test_task_packaged_when() {
+
+  Thread::TaskManager tm;
+  GMainLoop* loop = g_main_loop_new(0, true);
+  count = 0;
+
+  tm.make_task_packaged_when(
+    [loop] (std::future<int>& fut) -> void {
+      g_assert_cmpint(fut.get(), ==, 7);
+      mutex.lock();
+      ++count;
+      if (count == 2) g_main_loop_quit(loop);
+      mutex.unlock();
+    },
+    0,
+    G_PRIORITY_DEFAULT,
+    0,
+    [] () -> int {return 3 + 4;}
+  );
+
+  tm.make_task_packaged_when(
+    [loop] (std::future<int>& fut) -> void {
+      try {
+	int i = fut.get();
+	g_assert_not_reached();
+	++i;
+      }
+      catch (std::exception&) {
+	mutex.lock();
+	++count;
+	if (count == 2) g_main_loop_quit(loop);
+	mutex.unlock();
+      }
+    },
+    0,
+    [] () -> int {throw std::exception(); return 0;}
+  );
+  g_main_loop_run(loop);
+  mutex.lock();
+  g_assert_cmpuint(count, ==, 2);
+  mutex.unlock();
+
+  {
+    Releaser releaser;
+    tm.make_task_packaged_when(
+      [](std::future<int>&) -> void {g_assert_not_reached();},
+      &releaser,
+      G_PRIORITY_DEFAULT, 0,
+      []() -> int {return 0;}
+    );
+    tm.add_task([loop]() -> void {
+	g_usleep(100000);
+	Callback::post([loop] () -> void {
+	    g_main_loop_quit(loop);
+	  });
+      });
+  }
+  g_main_loop_run(loop);
+}
+
+static void test_task_packaged_compose() {
+
+  Thread::TaskManager tm;
+  GMainLoop* loop = g_main_loop_new(0, true);
+  count = 0;
+
+  tm.make_task_packaged_compose(
+    [] () -> std::string {return "Test";},
+    0,
+    [loop] (std::future<std::string>& fut) -> void {
+      g_assert(fut.get() == "Test");
+      mutex.lock();
+      ++count;
+      if (count == 2) g_main_loop_quit(loop);
+      mutex.unlock();
+    }
+  );
+
+  tm.make_task_packaged_compose(
+    [] () -> std::string {throw std::exception(); return "Test";},
+    0,
+    [loop] (std::future<std::string>& fut) -> void {
+      try {
+	std::string s = fut.get();
+	g_assert_not_reached();
+	s = "";
+      }
+      catch (std::exception&) {
+	mutex.lock();
+	++count;
+	if (count == 2) g_main_loop_quit(loop);
+	mutex.unlock();
+      }
+    }
+  );
+  g_main_loop_run(loop);
+  mutex.lock();
+  g_assert_cmpuint(count, ==, 2);
+  mutex.unlock();
+}
+
 static void test_inc_handle() {
 
   Thread::TaskManager tm{1};
@@ -816,8 +953,8 @@ static void test_inc_handle() {
   
   {
     Thread::TaskManager::IncHandle h{tm};
-    auto res1 = tm.make_task_result(&upause, 100000);
-    auto res2 = tm.make_task_result(&upause, 100000);
+    auto res1 = tm.make_task_result([] () -> int {g_usleep(100000); return 0;});
+    auto res2 = tm.make_task_result([] () -> int {g_usleep(100000); return 0;});
     res1->get();
     res2->get();
     g_assert_cmpuint(tm.get_used_threads(), ==, 2U);
@@ -841,9 +978,12 @@ int main (int argc, char* argv[]) {
   g_test_add_func("/task_manager/fail", test_fail); 
   g_test_add_func("/task_manager/idle", test_idle); 
   g_test_add_func("/task_manager/task_result", test_task_result); 
+  g_test_add_func("/task_manager/task_packaged", test_task_packaged); 
   g_test_add_func("/task_manager/task_when", test_task_when); 
   g_test_add_func("/task_manager/task_compose", test_task_compose); 
   g_test_add_func("/task_manager/task_when_full", test_task_when_full); 
+  g_test_add_func("/task_manager/task_packaged_when", test_task_packaged_when); 
+  g_test_add_func("/task_manager/task_packaged_compose", test_task_packaged_compose); 
   g_test_add_func("/task_manager/inc_handle", test_inc_handle); 
 
   return g_test_run();
